#--------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------
#
#            Examples for manuscript in Methods in Ecology and Evolution (2020):
# The multiple population genetic and demographic routes to islands of genomic divergence
#
# Example 1. Selection: recombination map and fitness function
# Example 2. Neutral evolution: Mutation rate, average recombination, biallelic SNPs,
#                               without fitness function.
# More examples in help(evolve)
#
# Questions to:
# Claudio S. Quilodrán. Department of Genetics and Evolution, University of Geneva.
#         Email: claudio.quilodran@unige.ch
# Eric C. Anderson. Department of Ecology and Evolutionary Biology, University of California.
#         Email: eric.anderson@noaa.gov
#--------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------

# Download the source package in your working directory
# Note that you may need to install 'Rtools' if it is not already installed
# install.packages("remotes") 
# remotes::install_local("glads_0.1.7.tar.gz")
library(glads)

# citation("glads")
#################################
## 	 Example 1: see Table 1    ##
##	   	recombination map,     ##
## 	 	with fitness function    ##
#################################
########################
### Generating data  ###
########################
initial.population.size = 400 #Initial population size
n.loci = 300					#Number of loci
n.alleles.per.locus = 20		#Number of alleles

set.seed(1)
start.1 <-
  start.2 <-
  initial.struct(initial.population.size, n.loci, n.alleles.per.locus)
start <- list(start.1, start.2) #Initial populations

###################
### Parameters  ###
###################
loci.pos <- 1:n.loci			#Loci position
add.loci <- 50					#Number of additive loci
fitness.pos <- 125:(125 + (add.loci - 1))   #Additive loci position
sex.ratio <- 0.5

bvs <- t(array(
  seq(0, 1, length = n.alleles.per.locus) ,
  c(n.alleles.per.locus, add.loci)
)) #Breeding value of additive loci
e.v <- 0.01 					#Stochastic environmental variant
d.v <- 1 						#Stochastic demographic variant
n.gens <- 100					#Number of generations
each <- 5						#Output stored each 5 generations


recom.map <- c(rep(0.5, 299))	#Recombination map
Linked <- c(60:69, 150:159, 230:239) #Linked loci
recom.map[Linked] <- 0.0001	#Recombination map with linked loci

#b0 Maximum generated offspring
#b1 Phenotypic optima
#b2 Variance of the fitness curve
#b3 Density-dependent demographic variant
##Parameters of the fitness function
param.w1 <-
  list(
    b0 = 6,
    b1 = 55,
    b2 = 5,
    b3 = 0.01,
    d.v = d.v,
    add.loci = add.loci
  )
param.w2 <-
  list(
    b0 = 6,
    b1 = 45,
    b2 = 5,
    b3 = 0.005,
    d.v = d.v,
    add.loci = add.loci
  )

##Parameters of the phenotype function
param.z1 <- list(sex.ratio, fitness.pos, bvs, add.loci, e.v)
param.z2 <- param.z1

#Check the choice of parameters
#z<-phenotype(start.1[, fitness.pos,], bvs, add.loci, sex.ratio, e.v)
#hist(z)
#fit1<-fitness(z, dim(start.1)[1], b0 = 6,b1 = 55, b2 = 5, b3 = 0.01, d.v = d.v)
#fit2<-fitness(z, dim(start.1)[1], b0 = 6,b1 = 45, b2 = 5, b3 = 0.01, d.v = d.v)
#sum(fit1[which(z[, "sex"]==2)]) #expected offspring
#sum(fit2[which(z[, "sex"]==2)]) #expected offspring

###################
### Simulations ###
###################
example <-
  evolve(
    x = start,
    time = c(n.gens, each),
    type = "additive",
    recombination = "map",
    recom.rate = recom.map,
    param.z = list(param.z1, param.z2),
    param.w = list(param.w1, param.w2)
  )


########################
### Population sizes ###
########################
npop <- 2
n.outputs <- (n.gens %/% each) + 1 #Number of recorded outputs
popsizes <- sapply(1:npop, function(i) {
  sapply(1:n.outputs, function(j) {
    dim(example[[j]][[i]])[1]
  })
})

plot(
  1:n.outputs,
  popsizes[, 1],
  type = "l",
  bty = "l",
  ylab = "Population size",
  xlab = "Generations",
  xaxs = "i",
  yaxs = "i",
  xaxt = "n",
  las = 1,
  ylim = c(0, max(popsizes)),
  lwd = 2,
  col = "darkred"
)
axis(1, labels = c(1, n.gens), at = c(1, n.outputs))

lines(1:n.outputs,
      popsizes[, 2],
      col = "darkgoldenrod",
      lwd = 2)
legend(
  "bottomright",
  legend = paste("Population", 1:2),
  col = c("darkred", "darkgoldenrod"),
  lwd = 2,
  bty = "n"
)


###################
###     Fst     ###
###################
##Commputation of Fst values for the  last generation
#We should first convert the output from class 'struct' to class 'loci'

sim.end <- n.outputs
  example.loci <- struct2pegas(example[[sim.end]])
  #example.loci <- struct2pegas(example[[paste(n.gens)]]) #same output

#Estimating Fst with library 'pegas'
require(pegas)
FST <- Fst(example.loci)

#Figure
plot(
  loci.pos,
  FST[, "Fst"],
  pch = 16,
  bty = "l",
  xlab = "Loci position",
  ylab = "Fst",
  ylim = c(0, 1),
  type = "l",
  xaxs = "i",
  yaxs = "i",
  las = 1,
  lwd = 2
)

######################################################################################################
######################################################################################################

#################################
## 	 Example 2: Mutation rate, ##
##	   average recombination,  ##
##		   biallelic SNPs      ##
## 	 without fitness function  ##
#################################
########################
### Generating data  ###
########################
Nind <- 200 	#Number of individuals
Nsnp <- 1000	#Number of SNPs
n.alleles <- 2

#Random genetic identity
set.seed(1)
start.1 <- start.2 <- initial.struct(Nind, Nsnp, n.alleles)

###################
### Parameters  ###
###################
chromo_mb <- 12000000 	 #Chromosome size
loci.pos <- sort(sample(1:chromo_mb, 1000, replace = F)) #Loci position

sex.ratio <- 0.5 			 #Sex ratio
mean.fitness <- 3 			 #Offspring per breeding pair
crossover <- 1.0 / 100000000.0 	 #Average recombination rate (cM/MB)
mutation.rate <- 1.1 * 10 ^ -8 	 #Mutation per site per generation
n.gens <- 100					 #Number of generations
migration.rate <- 0.075		 #Migration rate
d.d <- 0.005					 #Demographic effect to avoid exponential growth

param.z0 <- list(sex.ratio = sex.ratio)
param.w0 <- list(mean.fitness = mean.fitness, d.d = d.d)


###################
### Simulations ###
###################

example2 <-
  evolve(
    x = list(start.1, start.2),
    time = n.gens,
    type = "dynamic",
    recombination = "average",
    recom.rate = crossover,
    migration.rate = migration.rate,
    mutation.rate = mutation.rate,
    loci.pos = loci.pos,
    chromo_mb = chromo_mb,
    param.z = list(param.z0, param.z0),
    param.w = list(param.w0, param.w0)
  )


###################
###     Fst     ###
###################
##Commputation of Fst values
#We should first convert the output from class 'struct' to class 'loci'

example.loci2 <- struct2pegas(example2)

#Estimating Fst with library 'pegas'
require(pegas)
FST <- Fst(example.loci2)

#Figure
plot(
  loci.pos/1e+06,
  FST[, "Fst"],
  pch = 16,
  bty = "l",
  xlab = "Chromosome size (Mb)",
  ylab = "Fst",
  ylim = c(0, 1),
  type = "l",
  xaxs = "i",
  yaxs = "i",
  las = 1,
  lwd = 2
)
